import { _ as _extends, p as propTypes } from './index-c883ae0c.js';
import { _ as _objectWithoutPropertiesLoose } from './emotion-element-547f2a0a.esm-f471089a.js';
import { u as useUtils, i as isYearOnlyView, b as isYearAndMonthViews, P as PickersToolbar, m as makeValidationHook, c as useParsedDate, d as getFormatAndMaskByViews, v as validateDate, e as defaultMinDate, f as defaultMaxDate } from './moment-328d2707.js';
import { useMemo, createElement } from 'react';
import { c as clsx } from './clsx.m-6243b874.js';
import { w as withStyles } from './withStyles-22c017b8.js';
import { T as Typography } from './Typography-2dd3d2b2.js';
import { m as makePickerWithState, R as ResponsiveWrapper } from './makePickerWithState-fe269aa4.js';

const styles = {
  root: {},
  dateTitleLandscape: {
    margin: 'auto 16px auto auto'
  },
  penIcon: {
    position: 'relative',
    top: 4
  }
};
/**
 * @ignore - internal component.
 */

const DatePickerToolbar = (_ref) => {
  let {
    classes,
    date,
    isLandscape,
    isMobileKeyboardViewOpen,
    toggleMobileKeyboardView,
    toolbarFormat,
    toolbarPlaceholder = '––',
    toolbarTitle = 'SELECT DATE',
    views
  } = _ref,
      other = _objectWithoutPropertiesLoose(_ref, ["classes", "date", "isLandscape", "isMobileKeyboardViewOpen", "onChange", "toggleMobileKeyboardView", "toolbarFormat", "toolbarPlaceholder", "toolbarTitle", "views"]);

  const utils = useUtils();
  const dateText = useMemo(() => {
    if (!date) {
      return toolbarPlaceholder;
    }

    if (toolbarFormat) {
      return utils.formatByString(date, toolbarFormat);
    }

    if (isYearOnlyView(views)) {
      return utils.format(date, 'year');
    }

    if (isYearAndMonthViews(views)) {
      return utils.format(date, 'month');
    } // Little localization hack (Google is doing the same for android native pickers):
    // For english localization it is convenient to include weekday into the date "Mon, Jun 1".
    // For other locales using strings like "June 1", without weekday.


    return /en/.test(utils.getCurrentLocaleCode()) ? utils.format(date, 'normalDateWithWeekday') : utils.format(date, 'normalDate');
  }, [date, toolbarFormat, toolbarPlaceholder, utils, views]);
  return /*#__PURE__*/createElement(PickersToolbar, _extends({
    className: classes.root,
    toolbarTitle: toolbarTitle,
    isMobileKeyboardViewOpen: isMobileKeyboardViewOpen,
    toggleMobileKeyboardView: toggleMobileKeyboardView,
    isLandscape: isLandscape,
    penIconClassName: classes.penIcon
  }, other), /*#__PURE__*/createElement(Typography, {
    variant: "h4",
    align: isLandscape ? 'left' : 'center',
    className: clsx(isLandscape && classes.dateTitleLandscape)
  }, dateText));
};

var DatePickerToolbar$1 = withStyles(styles, {
  name: 'MuiDatePickerToolbar'
})(DatePickerToolbar);

const datePickerConfig = {
  useValidation: makeValidationHook(validateDate),
  DefaultToolbarComponent: DatePickerToolbar$1,
  useInterceptProps: (_ref) => {
    let {
      openTo = 'date',
      views = ['year', 'date'],
      minDate: __minDate = defaultMinDate,
      maxDate: __maxDate = defaultMaxDate
    } = _ref,
        other = _objectWithoutPropertiesLoose(_ref, ["openTo", "views", "minDate", "maxDate"]);

    const utils = useUtils();
    const minDate = useParsedDate(__minDate);
    const maxDate = useParsedDate(__maxDate);
    return _extends({
      views,
      openTo,
      minDate,
      maxDate
    }, getFormatAndMaskByViews(views, utils), other);
  }
};

/**
 *
 * Demos:
 *
 * - [Date Picker](https://material-ui.com/components/date-picker/)
 *
 * API:
 *
 * - [DatePicker API](https://material-ui.com/api/date-picker/)
 */
// @typescript-to-proptypes-generate
const DatePicker = makePickerWithState(ResponsiveWrapper, _extends({
  name: 'MuiDatePicker'
}, datePickerConfig));

if (process.env.NODE_ENV !== 'production') {
  DatePicker.displayName = 'DatePicker';
}

DatePicker.propTypes = {
  // ----------------------------- Warning --------------------------------
  // | These PropTypes are generated from the TypeScript type definitions |
  // |     To update them edit TypeScript types and run "yarn proptypes"  |
  // ----------------------------------------------------------------------

  /**
   * Regular expression to detect "accepted" symbols.
   * @default /\dap/gi
   */
  acceptRegex: propTypes.instanceOf(RegExp),

  /**
   * Cancel text message.
   * @default "CANCEL"
   */
  cancelText: propTypes.node,

  /**
   * @ignore
   */
  children: propTypes.node,

  /**
   * className applied to the root component.
   */
  className: propTypes.string,

  /**
   * If `true`, it shows the clear action in the picker dialog.
   * @default false
   */
  clearable: propTypes.bool,

  /**
   * Clear text message.
   * @default "CLEAR"
   */
  clearText: propTypes.node,

  /**
   * Props applied to the [`Dialog`](/api/dialog/) element.
   */
  DialogProps: propTypes.object,

  /**
   * If `true` the popup or dialog will immediately close after submitting full date.
   * @default `true` for Desktop, `false` for Mobile (based on the chosen wrapper and `desktopModeMediaQuery` prop).
   */
  disableCloseOnSelect: propTypes.bool,

  /**
   * If `true`, the picker and text field are disabled.
   */
  disabled: propTypes.bool,

  /**
   * Disable mask on the keyboard, this should be used rarely. Consider passing proper mask for your format.
   * @default false
   */
  disableMaskedInput: propTypes.bool,

  /**
   * Do not render open picker button (renders only text field with validation).
   * @default false
   */
  disableOpenPicker: propTypes.bool,

  /**
   * Get aria-label text for control that opens picker dialog. Aria-label text must include selected date. @DateIOType
   * @default (value, utils) => `Choose date, selected date is ${utils.format(utils.date(value), 'fullDate')}`
   */
  getOpenDialogAriaText: propTypes.func,

  /**
   * @ignore
   */
  ignoreInvalidInputs: propTypes.bool,

  /**
   * Props to pass to keyboard input adornment.
   */
  InputAdornmentProps: propTypes.object,

  /**
   * Format string.
   */
  inputFormat: propTypes.string,

  /**
   * @ignore
   */
  InputProps: propTypes.object,

  /**
   * @ignore
   */
  key: propTypes.oneOfType([propTypes.number, propTypes.string]),

  /**
   * @ignore
   */
  label: propTypes.node,

  /**
   * Custom mask. Can be used to override generate from format. (e.g. `__/__/____ __:__` or `__/__/____ __:__ _M`).
   */
  mask: propTypes.string,

  /**
   * @ignore
   */
  maxDate: propTypes.oneOfType([propTypes.any, propTypes.instanceOf(Date), propTypes.number, propTypes.string]),

  /**
   * @ignore
   */
  minDate: propTypes.oneOfType([propTypes.any, propTypes.instanceOf(Date), propTypes.number, propTypes.string]),

  /**
   * Ok button text.
   * @default "OK"
   */
  okText: propTypes.node,

  /**
   * Callback fired when date is accepted @DateIOType.
   */
  onAccept: propTypes.func,

  /**
   * Callback fired when the value (the selected date) changes @DateIOType.
   */
  onChange: propTypes.func.isRequired,

  /**
   * Callback fired when the popup requests to be closed.
   * Use in controlled mode (see open).
   */
  onClose: propTypes.func,

  /**
   * Callback that fired when input value or new `value` prop validation returns **new** validation error (or value is valid after error).
   * In case of validation error detected `reason` prop return non-null value and `TextField` must be displayed in `error` state.
   * This can be used to render appropriate form error.
   *
   * [Read the guide](https://next.material-ui-pickers.dev/guides/forms) about form integration and error displaying.
   * @DateIOType
   */
  onError: propTypes.func,

  /**
   * Callback fired when the popup requests to be opened.
   * Use in controlled mode (see open).
   */
  onOpen: propTypes.func,

  /**
   * Control the popup or dialog open state.
   */
  open: propTypes.bool,

  /**
   * Props to pass to keyboard adornment button.
   */
  OpenPickerButtonProps: propTypes.object,

  /**
   * Icon displaying for open picker button.
   */
  openPickerIcon: propTypes.node,

  /**
   * Force rendering in particular orientation.
   */
  orientation: propTypes.oneOf(['landscape', 'portrait']),

  /**
   * Make picker read only.
   */
  readOnly: propTypes.bool,

  /**
   * The `renderInput` prop allows you to customize the rendered input.
   * The `props` argument of this render prop contains props of [TextField](https://material-ui.com/api/text-field/#textfield-api) that you need to forward.
   * Pay specific attention to the `ref` and `inputProps` keys.
   * @example ```jsx
   * renderInput={props => <TextField {...props} />}
   * ````
   */
  renderInput: propTypes.func.isRequired,

  /**
   * Custom formatter to be passed into Rifm component.
   */
  rifmFormatter: propTypes.func,

  /**
   * If `true`, the today button is displayed. **Note** that `showClearButton` has a higher priority.
   * @default false
   */
  showTodayButton: propTypes.bool,

  /**
   * If `true`, show the toolbar even in desktop mode.
   */
  showToolbar: propTypes.bool,

  /**
   * Today text message.
   * @default "TODAY"
   */
  todayText: propTypes.node,

  /**
   * Component that will replace default toolbar renderer.
   */
  ToolbarComponent: propTypes.elementType,

  /**
   * Date format, that is displaying in toolbar.
   */
  toolbarFormat: propTypes.string,

  /**
   * Mobile picker date value placeholder, displaying if `value` === `null`.
   * @default "–"
   */
  toolbarPlaceholder: propTypes.node,

  /**
   * Mobile picker title, displaying in the toolbar.
   * @default "SELECT DATE"
   */
  toolbarTitle: propTypes.node,

  /**
   * The value of the picker.
   */
  value: propTypes.oneOfType([propTypes.any, propTypes.instanceOf(Date), propTypes.number, propTypes.string])
};

export { DatePicker as D };
//# sourceMappingURL=DatePicker-9caa2e52.js.map
