import { _ as _extends, p as propTypes } from './index-8b5bbbf1.js';
import { _ as _objectWithoutPropertiesLoose } from './objectWithoutPropertiesLoose-6856ddd5.js';
import { forwardRef, createElement, Fragment } from 'react';
import { c as clsx } from './clsx.m-6243b874.js';
import { w as withStyles } from './withStyles-6980ffec.js';
import { u as useFormControl, F as FormControlContext } from './FormControlContext-3e99c910.js';
import { T as Typography } from './Typography-c34df849.js';

const styles = theme => ({
  /* Styles applied to the root element. */
  root: {
    display: 'flex',
    height: '0.01em',
    // Fix IE11 flexbox alignment. To remove at some point.
    maxHeight: '2em',
    alignItems: 'center',
    whiteSpace: 'nowrap',
    color: theme.palette.action.active
  },

  /* Styles applied to the root element if `variant="filled"`. */
  filled: {
    '&$positionStart:not($hiddenLabel)': {
      marginTop: 16
    }
  },

  /* Styles applied to the root element if `position="start"`. */
  positionStart: {
    marginRight: 8
  },

  /* Styles applied to the root element if `position="end"`. */
  positionEnd: {
    marginLeft: 8
  },

  /* Styles applied to the root element if `disablePointerEvents={true}`. */
  disablePointerEvents: {
    pointerEvents: 'none'
  },

  /* Styles applied if the adornment is used inside <FormControl hiddenLabel />. */
  hiddenLabel: {},

  /* Styles applied if the adornment is used inside <FormControl size="small" />. */
  sizeSmall: {}
});
const InputAdornment = /*#__PURE__*/forwardRef(function InputAdornment(props, ref) {
  const {
    children,
    classes,
    className,
    component: Component = 'div',
    disablePointerEvents = false,
    disableTypography = false,
    position,
    variant: variantProp
  } = props,
        other = _objectWithoutPropertiesLoose(props, ["children", "classes", "className", "component", "disablePointerEvents", "disableTypography", "position", "variant"]);

  const muiFormControl = useFormControl() || {};
  let variant = variantProp;

  if (variantProp && muiFormControl.variant) {
    if (process.env.NODE_ENV !== 'production') {
      if (variantProp === muiFormControl.variant) {
        console.error('Material-UI: The `InputAdornment` variant infers the variant prop ' + 'you do not have to provide one.');
      }
    }
  }

  if (muiFormControl && !variant) {
    variant = muiFormControl.variant;
  }

  return /*#__PURE__*/createElement(FormControlContext.Provider, {
    value: null
  }, /*#__PURE__*/createElement(Component, _extends({
    className: clsx(classes.root, className, disablePointerEvents && classes.disablePointerEvents, muiFormControl.hiddenLabel && classes.hiddenLabel, variant === 'filled' && classes.filled, {
      'start': classes.positionStart,
      'end': classes.positionEnd
    }[position], muiFormControl.size === 'small' && classes.sizeSmall),
    ref: ref
  }, other), typeof children === 'string' && !disableTypography ? /*#__PURE__*/createElement(Typography, {
    color: "textSecondary"
  }, children) : /*#__PURE__*/createElement(Fragment, null, position === 'start' ?
  /*#__PURE__*/

  /* notranslate needed while Google Translate will not fix zero-width space issue */

  /* eslint-disable-next-line react/no-danger */
  createElement("span", {
    className: "notranslate",
    dangerouslySetInnerHTML: {
      __html: '&#8203;'
    }
  }) : null, children)));
});
process.env.NODE_ENV !== "production" ? InputAdornment.propTypes = {
  // ----------------------------- Warning --------------------------------
  // | These PropTypes are generated from the TypeScript type definitions |
  // |     To update them edit the d.ts file and run "yarn proptypes"     |
  // ----------------------------------------------------------------------

  /**
   * The content of the component, normally an `IconButton` or string.
   */
  children: propTypes.node,

  /**
   * Override or extend the styles applied to the component.
   */
  classes: propTypes.object,

  /**
   * @ignore
   */
  className: propTypes.string,

  /**
   * The component used for the root node.
   * Either a string to use a HTML element or a component.
   */
  component: propTypes.elementType,

  /**
   * Disable pointer events on the root.
   * This allows for the content of the adornment to focus the `input` on click.
   * @default false
   */
  disablePointerEvents: propTypes.bool,

  /**
   * If children is a string then disable wrapping in a Typography component.
   * @default false
   */
  disableTypography: propTypes.bool,

  /**
   * The position this adornment should appear relative to the `Input`.
   */
  position: propTypes.oneOf(['end', 'start']),

  /**
   * The variant to use.
   * Note: If you are using the `TextField` component or the `FormControl` component
   * you do not have to set this manually.
   */
  variant: propTypes.oneOf(['filled', 'outlined', 'standard'])
} : void 0;
var InputAdornment$1 = withStyles(styles, {
  name: 'MuiInputAdornment'
})(InputAdornment);

export { InputAdornment$1 as I };
//# sourceMappingURL=InputAdornment-e69e599f.js.map
