import { _ as __assign } from './tslib.es6-a941f97f.js';
import { jsx, jsxs } from 'react/jsx-runtime';
import React, { forwardRef, createElement, Fragment } from 'react';
import { s as styles } from './EnhancedTable.module-3f17e3a7.js';
import ColumnDisplay from './ReactTable/Toolbar/ColumnDisplay.js';
import FilterBox from './ReactTable/Toolbar/FilterBox.js';
import { G as GlobalFilter } from './GlobalFilter-e23922bd.js';
import LocalGlobalFilter from './ReactTable/Toolbar/LocalGlobalFilter.js';
import { _ as _objectWithoutPropertiesLoose } from './objectWithoutPropertiesLoose-6856ddd5.js';
import { _ as _extends } from './extends-e61fd370.js';
import { p as propTypes } from './index-df27ca20.js';
import { c as clsx } from './clsx.m-6243b874.js';
import { m as generateUtilityClass, d as useThemeProps, e as deepmerge } from './emotion-element-547f2a0a.esm-194d172b.js';
import { g as generateUtilityClasses, e as experimentalStyled, c as composeClasses } from './experimentalStyled-d34ee85b.js';

function getToolbarUtilityClass(slot) {
  return generateUtilityClass('MuiToolbar', slot);
}
generateUtilityClasses('MuiToolbar', ['root', 'gutters', 'regular', 'dense']);

const overridesResolver = (props, styles) => {
  const {
    styleProps
  } = props;
  return deepmerge(_extends({}, !styleProps.disableGutters && styles.gutters, styles[styleProps.variant]), styles.root || {});
};

const useUtilityClasses = styleProps => {
  const {
    classes,
    disableGutters,
    variant
  } = styleProps;
  const slots = {
    root: ['root', !disableGutters && 'gutters', variant]
  };
  return composeClasses(slots, getToolbarUtilityClass, classes);
};

const ToolbarRoot = experimentalStyled('div', {}, {
  name: 'MuiToolbar',
  slot: 'Root',
  overridesResolver
})(({
  theme,
  styleProps
}) => _extends({
  /* Styles applied to the root element. */
  position: 'relative',
  display: 'flex',
  alignItems: 'center'
}, !styleProps.disableGutters && {
  paddingLeft: theme.spacing(2),
  paddingRight: theme.spacing(2),
  [theme.breakpoints.up('sm')]: {
    paddingLeft: theme.spacing(3),
    paddingRight: theme.spacing(3)
  }
}, styleProps.variant === 'dense' && {
  minHeight: 48
}),
/* Styles applied to the root element if `variant="regular"`. */
({
  theme,
  styleProps
}) => styleProps.variant === 'regular' && theme.mixins.toolbar);
const Toolbar = /*#__PURE__*/forwardRef(function Toolbar(inProps, ref) {
  const props = useThemeProps({
    props: inProps,
    name: 'MuiToolbar'
  });

  const {
    className,
    component = 'div',
    disableGutters = false,
    variant = 'regular'
  } = props,
        other = _objectWithoutPropertiesLoose(props, ["className", "component", "disableGutters", "variant"]);

  const styleProps = _extends({}, props, {
    component,
    disableGutters,
    variant
  });

  const classes = useUtilityClasses(styleProps);
  return /*#__PURE__*/createElement(ToolbarRoot, _extends({
    as: component,
    className: clsx(classes.root, className),
    ref: ref,
    styleProps: styleProps
  }, other));
});
process.env.NODE_ENV !== "production" ? Toolbar.propTypes = {
  // ----------------------------- Warning --------------------------------
  // | These PropTypes are generated from the TypeScript type definitions |
  // |     To update them edit the d.ts file and run "yarn proptypes"     |
  // ----------------------------------------------------------------------

  /**
   * The Toolbar children, usually a mixture of `IconButton`, `Button` and `Typography`.
   * The Toolbar is a flex container, allowing flex item properites to be used to lay out the children.
   */
  children: propTypes.node,

  /**
   * Override or extend the styles applied to the component.
   */
  classes: propTypes.object,

  /**
   * @ignore
   */
  className: propTypes.string,

  /**
   * The component used for the root node.
   * Either a string to use a HTML element or a component.
   */
  component: propTypes.elementType,

  /**
   * If `true`, disables gutter padding.
   * @default false
   */
  disableGutters: propTypes.bool,

  /**
   * The system prop that allows defining system overrides as well as additional CSS styles.
   */
  sx: propTypes.object,

  /**
   * The variant to use.
   * @default 'regular'
   */
  variant: propTypes
  /* @typescript-to-proptypes-ignore */
  .oneOfType([propTypes.oneOf(['dense', 'regular']), propTypes.string])
} : void 0;

var TableToolbar = function (props) {
    var _a = props.isLocal, isLocal = _a === void 0 ? false : _a, _b = props.hasGlobalSearch, hasGlobalSearch = _b === void 0 ? true : _b, defaultHiddenColumns = props.defaultHiddenColumns;
    var selectedRowNumber = props.instance.selectedFlatRows.length;
    return (jsx(Toolbar, __assign({ className: styles.toolbar, style: { width: props.width } }, { children: jsxs(Fragment, { children: [jsxs("div", __assign({ className: styles.actionBox }, { children: [props.toolbarItems, jsx(FilterBox, { instance: props.instance, stableColumns: props.stableColumns }, void 0),
                        jsx(ColumnDisplay, { instance: props.instance, stableColumns: props.stableColumns, defaultHiddenColumns: defaultHiddenColumns }, void 0),
                        hasGlobalSearch && isLocal && (jsx(LocalGlobalFilter, { globalFilter: props.instance.state.globalFilter, setGlobalFilter: props.instance.setGlobalFilter }, void 0)),
                        hasGlobalSearch && !isLocal && jsx(GlobalFilter, {}, void 0),
                        selectedRowNumber > 0 && (jsxs("div", __assign({ className: styles.infoText }, { children: [selectedRowNumber, " Selected Rows"] }), void 0))] }), void 0), props.pagination] }, void 0) }), void 0));
};
var TableToolbar$1 = React.memo(TableToolbar);

export { TableToolbar$1 as T, Toolbar as a };
//# sourceMappingURL=TableToolbar-23e1ca1e.js.map
