import { _ as _extends, p as propTypes } from './index-c883ae0c.js';
import { u as useTheme, G as getThemeProps, _ as _objectWithoutPropertiesLoose, e as useThemeProps } from './emotion-element-547f2a0a.esm-f471089a.js';
import { useState, useDebugValue, useRef, useEffect, createElement, forwardRef } from 'react';
import { h as useGlobalKeyDown, k as keycode, W as WrapperVariantContext, K as KeyboardDateInput, M as MobileWrapper, j as usePickerState, l as Picker, n as makeWrapperComponent, p as parsePickerInputValue, o as PureDateInput } from './moment-328d2707.js';
import { c as clsx } from './clsx.m-6243b874.js';
import { w as withStyles } from './withStyles-22c017b8.js';
import { P as Popper } from './Popper-bbe7b45f.js';
import { U as Unstable_TrapFocus, P as Paper } from './Popover-ca3d9161.js';
import { G as Grow } from './Grow-1c55ca27.js';
import { u as useEnhancedEffect, f as useForkRef, d as useEventCallback } from './experimentalStyled-899ca16e.js';
import { o as ownerDocument } from './ownerDocument-08286491.js';

function useMediaQuery(queryInput, options = {}) {
  const theme = useTheme();
  const props = getThemeProps({
    theme,
    name: 'MuiUseMediaQuery',
    props: {}
  });

  if (process.env.NODE_ENV !== 'production') {
    if (typeof queryInput === 'function' && theme === null) {
      console.error(['Material-UI: The `query` argument provided is invalid.', 'You are providing a function without a theme in the context.', 'One of the parent elements needs to use a ThemeProvider.'].join('\n'));
    }
  }

  let query = typeof queryInput === 'function' ? queryInput(theme) : queryInput;
  query = query.replace(/^@media( ?)/m, ''); // Wait for jsdom to support the match media feature.
  // All the browsers Material-UI support have this built-in.
  // This defensive check is here for simplicity.
  // Most of the time, the match media logic isn't central to people tests.

  const supportMatchMedia = typeof window !== 'undefined' && typeof window.matchMedia !== 'undefined';

  const {
    defaultMatches = false,
    matchMedia = supportMatchMedia ? window.matchMedia : null,
    noSsr = false,
    ssrMatchMedia = null
  } = _extends({}, props, options);

  const [match, setMatch] = useState(() => {
    if (noSsr && supportMatchMedia) {
      return matchMedia(query).matches;
    }

    if (ssrMatchMedia) {
      return ssrMatchMedia(query).matches;
    } // Once the component is mounted, we rely on the
    // event listeners to return the correct matches value.


    return defaultMatches;
  });
  useEnhancedEffect(() => {
    let active = true;

    if (!supportMatchMedia) {
      return undefined;
    }

    const queryList = matchMedia(query);

    const updateMatch = () => {
      // Workaround Safari wrong implementation of matchMedia
      // TODO can we remove it?
      // https://github.com/mui-org/material-ui/pull/17315#issuecomment-528286677
      if (active) {
        setMatch(queryList.matches);
      }
    };

    updateMatch();
    queryList.addListener(updateMatch);
    return () => {
      active = false;
      queryList.removeListener(updateMatch);
    };
  }, [query, matchMedia, supportMatchMedia]);

  if (process.env.NODE_ENV !== 'production') {
    // eslint-disable-next-line react-hooks/rules-of-hooks
    useDebugValue({
      query,
      match
    });
  }

  return match;
}

const styles = theme => ({
  root: {
    zIndex: theme.zIndex.modal
  },
  paper: {
    transformOrigin: 'top center',
    outline: 0
  },
  topTransition: {
    transformOrigin: 'bottom center'
  }
});

function clickedRootScrollbar(event, doc) {
  return doc.documentElement.clientWidth < event.clientX || doc.documentElement.clientHeight < event.clientY;
}
/**
 * Based on @material-ui/core/ClickAwayListener without the customization.
 * We can probably strip away even more since children won't be portaled.
 *
 * @param onClickAway
 * @param onClick
 * @param onTouchStart
 */


function useClickAwayListener(active, onClickAway) {
  const movedRef = useRef(false);
  const syntheticEventRef = useRef(false);
  const nodeRef = useRef(null);
  const activatedRef = useRef(false);
  useEffect(() => {
    if (!active) {
      return undefined;
    }

    function handleClickCapture() {
      activatedRef.current = true;
    }

    document.addEventListener('click', handleClickCapture, {
      capture: true,
      once: true
    });
    return () => {
      activatedRef.current = false;
      document.removeEventListener('click', handleClickCapture, {
        capture: true
      });
    };
  }, [active]); // The handler doesn't take event.defaultPrevented into account:
  //
  // event.preventDefault() is meant to stop default behaviors like
  // clicking a checkbox to check it, hitting a button to submit a form,
  // and hitting left arrow to move the cursor in a text input etc.
  // Only special HTML elements have these default behaviors.

  const handleClickAway = useEventCallback(event => {
    if (!activatedRef.current) {
      return;
    } // Given developers can stop the propagation of the synthetic event,
    // we can only be confident with a positive value.


    const insideReactTree = syntheticEventRef.current;
    syntheticEventRef.current = false;
    const doc = ownerDocument(nodeRef.current); // 1. IE11 support, which trigger the handleClickAway even after the unbind
    // 2. The child might render null.
    // 3. Behave like a blur listener.

    if (!nodeRef.current || // is a TouchEvent?
    'clientX' in event && clickedRootScrollbar(event, doc)) {
      return;
    } // Do not act if user performed touchmove


    if (movedRef.current) {
      movedRef.current = false;
      return;
    }

    let insideDOM; // If not enough, can use https://github.com/DieterHolvoet/event-propagation-path/blob/master/propagationPath.js

    if (event.composedPath) {
      insideDOM = event.composedPath().indexOf(nodeRef.current) > -1;
    } else {
      insideDOM = !doc.documentElement.contains(event.target) || nodeRef.current.contains(event.target);
    }

    if (!insideDOM && !insideReactTree) {
      onClickAway(event);
    }
  }); // Keep track of mouse/touch events that bubbled up through the portal.

  const handleSynthetic = () => {
    syntheticEventRef.current = true;
  };

  useEffect(() => {
    if (active) {
      const doc = ownerDocument(nodeRef.current);

      const handleTouchMove = () => {
        movedRef.current = true;
      };

      doc.addEventListener('touchstart', handleClickAway);
      doc.addEventListener('touchmove', handleTouchMove);
      return () => {
        doc.removeEventListener('touchstart', handleClickAway);
        doc.removeEventListener('touchmove', handleTouchMove);
      };
    }

    return undefined;
  }, [active, handleClickAway]);
  useEffect(() => {
    // TODO This behavior is not tested automatically
    // It's unclear whether this is due to different update semantics in test (batched in act() vs discrete on click).
    // Or if this is a timing related issues due to different Transition components
    // Once we get rid of all the manual scheduling (e.g. setTimeout(update, 0)) we can revisit this code+test.
    if (active) {
      const doc = ownerDocument(nodeRef.current);
      doc.addEventListener('click', handleClickAway);
      return () => {
        doc.removeEventListener('click', handleClickAway);
      };
    }

    return undefined;
  }, [active, handleClickAway]);
  return [nodeRef, handleSynthetic, handleSynthetic];
}

const PickersPopper = props => {
  const {
    anchorEl,
    children,
    classes,
    containerRef = null,
    onClose,
    open,
    PopperProps,
    role,
    TransitionComponent = Grow,
    TrapFocusProps
  } = props;
  useGlobalKeyDown(open, {
    [keycode.Esc]: onClose
  });
  const lastFocusedElementRef = useRef(null);
  useEffect(() => {
    if (role === 'tooltip') {
      return;
    }

    if (open) {
      lastFocusedElementRef.current = document.activeElement;
    } else if (lastFocusedElementRef.current && lastFocusedElementRef.current instanceof HTMLElement) {
      lastFocusedElementRef.current.focus();
    }
  }, [open, role]);
  const [clickAwayRef, onPaperClick, onPaperTouchStart] = useClickAwayListener(open, onClose);
  const paperRef = useRef(null);
  const handleRef = useForkRef(paperRef, containerRef);
  const handlePaperRef = useForkRef(handleRef, clickAwayRef);
  return /*#__PURE__*/createElement(Popper, _extends({
    transition: true,
    role: role,
    open: open,
    anchorEl: anchorEl,
    className: clsx(classes.root, PopperProps === null || PopperProps === void 0 ? void 0 : PopperProps.className)
  }, PopperProps), ({
    TransitionProps,
    placement
  }) => /*#__PURE__*/createElement(Unstable_TrapFocus, _extends({
    open: open,
    disableAutoFocus: true,
    disableEnforceFocus: role === 'tooltip',
    isEnabled: () => true,
    getDoc: () => {
      var _paperRef$current$own, _paperRef$current;

      return (_paperRef$current$own = (_paperRef$current = paperRef.current) === null || _paperRef$current === void 0 ? void 0 : _paperRef$current.ownerDocument) !== null && _paperRef$current$own !== void 0 ? _paperRef$current$own : document;
    }
  }, TrapFocusProps), /*#__PURE__*/createElement(TransitionComponent, TransitionProps, /*#__PURE__*/createElement(Paper, {
    tabIndex: -1,
    elevation: 8,
    ref: handlePaperRef,
    className: clsx(classes.paper, placement === 'top' && classes.topTransition),
    onClick: onPaperClick,
    onTouchStart: onPaperTouchStart
  }, children))));
};

var PickersPopper$1 = withStyles(styles, {
  name: 'MuiPickersPopper'
})(PickersPopper);

const DesktopWrapper = props => {
  const {
    children,
    DateInputProps,
    KeyboardDateInputComponent = KeyboardDateInput,
    onDismiss,
    open,
    PopperProps,
    TransitionComponent
  } = props;
  const inputRef = useRef(null);
  return /*#__PURE__*/createElement(WrapperVariantContext.Provider, {
    value: "desktop"
  }, /*#__PURE__*/createElement(KeyboardDateInputComponent, _extends({}, DateInputProps, {
    inputRef: inputRef
  })), /*#__PURE__*/createElement(PickersPopper$1, {
    role: "dialog",
    open: open,
    anchorEl: inputRef.current,
    TransitionComponent: TransitionComponent,
    PopperProps: PopperProps,
    onClose: onDismiss
  }, children));
};

process.env.NODE_ENV !== "production" ? DesktopWrapper.propTypes = {
  onOpen: propTypes.func,
  onClose: propTypes.func
} : void 0;

const makeResponsiveWrapper = (DesktopWrapperComponent, MobileWrapperComponent) => {
  const ResponsiveWrapper = (_ref) => {
    let {
      cancelText,
      clearable,
      clearText,
      desktopModeMediaQuery = '@media (pointer: fine)',
      DialogProps,
      okText,
      PopperProps,
      showTodayButton,
      todayText,
      TransitionComponent
    } = _ref,
        other = _objectWithoutPropertiesLoose(_ref, ["cancelText", "clearable", "clearText", "desktopModeMediaQuery", "DialogProps", "okText", "PopperProps", "showTodayButton", "todayText", "TransitionComponent"]);

    const isDesktop = useMediaQuery(desktopModeMediaQuery);
    return isDesktop ? /*#__PURE__*/createElement(DesktopWrapperComponent, _extends({
      PopperProps: PopperProps,
      TransitionComponent: TransitionComponent
    }, other)) : /*#__PURE__*/createElement(MobileWrapperComponent, _extends({
      okText: okText,
      cancelText: cancelText,
      clearText: clearText,
      todayText: todayText,
      showTodayButton: showTodayButton,
      clearable: clearable,
      DialogProps: DialogProps
    }, other));
  };

  return ResponsiveWrapper;
};
const ResponsiveWrapper = makeResponsiveWrapper(DesktopWrapper, MobileWrapper);

const valueManager = {
  emptyValue: null,
  parseInput: parsePickerInputValue,
  areValuesEqual: (utils, a, b) => utils.isEqual(a, b)
};
function makePickerWithState(Wrapper, {
  name,
  useInterceptProps,
  useValidation,
  DefaultToolbarComponent
}) {
  const WrapperComponent = makeWrapperComponent(Wrapper, {
    KeyboardDateInputComponent: KeyboardDateInput,
    PureDateInputComponent: PureDateInput
  });
  const PickerWithState = /*#__PURE__*/forwardRef(function PickerWithState(__props, ref) {
    const allProps = useInterceptProps(__props); // This is technically unsound if the type parameters appear in optional props.
    // Optional props can be filled by `useThemeProps` with types that don't match the type parameters.

    const props = useThemeProps({
      props: allProps,
      name
    });
    const validationError = useValidation(props.value, props) !== null;
    const {
      pickerProps,
      inputProps,
      wrapperProps
    } = usePickerState(props, valueManager); // Note that we are passing down all the value without spread.
    // It saves us >1kb gzip and make any prop available automatically on any level down.

    const other = _objectWithoutPropertiesLoose(props, ["value", "onChange"]);

    const AllDateInputProps = _extends({}, inputProps, other, {
      ref,
      validationError
    });

    return /*#__PURE__*/createElement(WrapperComponent, _extends({
      wrapperProps: wrapperProps,
      DateInputProps: AllDateInputProps
    }, other), /*#__PURE__*/createElement(Picker, _extends({}, pickerProps, {
      toolbarTitle: props.label || props.toolbarTitle,
      ToolbarComponent: other.ToolbarComponent || DefaultToolbarComponent,
      DateInputProps: AllDateInputProps
    }, other)));
  }); // @ts-expect-error Types are equal except `forwardRef` calls the returned types with `PropsWithoutRef`.
  // The distributive nature of `PropsWithOutRef` causes the type error.
  // TODO: Find out why we need a distributive `PropsWithOutRef`.

  return PickerWithState;
}

export { ResponsiveWrapper as R, makePickerWithState as m };
//# sourceMappingURL=makePickerWithState-fe269aa4.js.map
