import { useContext, useDebugValue, useReducer, useMemo, useRef, forwardRef, createElement, useState, useEffect } from 'react';
import { a as useReduxContext } from './useDispatch-07bebf84.js';
import { S as Subscription, R as ReactReduxContext } from './index-23f55da3.js';
import { u as useIsomorphicLayoutEffect } from './WithApp-66cc801a.js';
import { _ as _extends } from './extends-e61fd370.js';
import { _ as _objectWithoutPropertiesLoose } from './objectWithoutPropertiesLoose-6856ddd5.js';
import { p as propTypes } from './index-df27ca20.js';
import { c as clsx } from './clsx.m-6243b874.js';
import { m as generateUtilityClass, d as useThemeProps } from './emotion-element-547f2a0a.esm-194d172b.js';
import { g as generateUtilityClasses, e as experimentalStyled, c as composeClasses } from './experimentalStyled-d34ee85b.js';
import { R as ResizeObserver_2 } from './ResizeObserver-da68295b.js';

var refEquality = function refEquality(a, b) {
  return a === b;
};

function useSelectorWithStoreAndSubscription(selector, equalityFn, store, contextSub) {
  var _useReducer = useReducer(function (s) {
    return s + 1;
  }, 0),
      forceRender = _useReducer[1];

  var subscription = useMemo(function () {
    return new Subscription(store, contextSub);
  }, [store, contextSub]);
  var latestSubscriptionCallbackError = useRef();
  var latestSelector = useRef();
  var latestStoreState = useRef();
  var latestSelectedState = useRef();
  var storeState = store.getState();
  var selectedState;

  try {
    if (selector !== latestSelector.current || storeState !== latestStoreState.current || latestSubscriptionCallbackError.current) {
      selectedState = selector(storeState);
    } else {
      selectedState = latestSelectedState.current;
    }
  } catch (err) {
    if (latestSubscriptionCallbackError.current) {
      err.message += "\nThe error may be correlated with this previous error:\n" + latestSubscriptionCallbackError.current.stack + "\n\n";
    }

    throw err;
  }

  useIsomorphicLayoutEffect(function () {
    latestSelector.current = selector;
    latestStoreState.current = storeState;
    latestSelectedState.current = selectedState;
    latestSubscriptionCallbackError.current = undefined;
  });
  useIsomorphicLayoutEffect(function () {
    function checkForUpdates() {
      try {
        var newSelectedState = latestSelector.current(store.getState());

        if (equalityFn(newSelectedState, latestSelectedState.current)) {
          return;
        }

        latestSelectedState.current = newSelectedState;
      } catch (err) {
        // we ignore all errors here, since when the component
        // is re-rendered, the selectors are called again, and
        // will throw again, if neither props nor store state
        // changed
        latestSubscriptionCallbackError.current = err;
      }

      forceRender();
    }

    subscription.onStateChange = checkForUpdates;
    subscription.trySubscribe();
    checkForUpdates();
    return function () {
      return subscription.tryUnsubscribe();
    };
  }, [store, subscription]);
  return selectedState;
}
/**
 * Hook factory, which creates a `useSelector` hook bound to a given context.
 *
 * @param {React.Context} [context=ReactReduxContext] Context passed to your `<Provider>`.
 * @returns {Function} A `useSelector` hook bound to the specified context.
 */


function createSelectorHook(context) {
  if (context === void 0) {
    context = ReactReduxContext;
  }

  var useReduxContext$1 = context === ReactReduxContext ? useReduxContext : function () {
    return useContext(context);
  };
  return function useSelector(selector, equalityFn) {
    if (equalityFn === void 0) {
      equalityFn = refEquality;
    }

    if (process.env.NODE_ENV !== 'production' && !selector) {
      throw new Error("You must pass a selector to useSelector");
    }

    var _useReduxContext = useReduxContext$1(),
        store = _useReduxContext.store,
        contextSub = _useReduxContext.subscription;

    var selectedState = useSelectorWithStoreAndSubscription(selector, equalityFn, store, contextSub);
    useDebugValue(selectedState);
    return selectedState;
  };
}
/**
 * A hook to access the redux store's state. This hook takes a selector function
 * as an argument. The selector is called with the store state.
 *
 * This hook takes an optional equality comparison function as the second parameter
 * that allows you to customize the way the selected state is compared to determine
 * whether the component needs to be re-rendered.
 *
 * @param {Function} selector the selector function
 * @param {Function=} equalityFn the function that will be used to determine equality
 *
 * @returns {any} the selected state
 *
 * @example
 *
 * import React from 'react'
 * import { useSelector } from 'react-redux'
 *
 * export const CounterComponent = () => {
 *   const counter = useSelector(state => state.counter)
 *   return <div>{counter}</div>
 * }
 */

var useSelector = /*#__PURE__*/createSelectorHook();

function getTableContainerUtilityClass(slot) {
  return generateUtilityClass('MuiTableContainer', slot);
}
generateUtilityClasses('MuiTableContainer', ['root']);

const overridesResolver = (props, styles) => styles.root || {};

const useUtilityClasses = styleProps => {
  const {
    classes
  } = styleProps;
  const slots = {
    root: ['root']
  };
  return composeClasses(slots, getTableContainerUtilityClass, classes);
};

const TableContainerRoot = experimentalStyled('div', {}, {
  name: 'MuiTableContainer',
  slot: 'Root',
  overridesResolver
})({
  /* Styles applied to the root element. */
  width: '100%',
  overflowX: 'auto'
});
const TableContainer = /*#__PURE__*/forwardRef(function TableContainer(inProps, ref) {
  const props = useThemeProps({
    props: inProps,
    name: 'MuiTableContainer'
  });

  const {
    className,
    component = 'div'
  } = props,
        other = _objectWithoutPropertiesLoose(props, ["className", "component"]);

  const styleProps = _extends({}, props, {
    component
  });

  const classes = useUtilityClasses(styleProps);
  return /*#__PURE__*/createElement(TableContainerRoot, _extends({
    ref: ref,
    as: component,
    className: clsx(classes.root, className),
    styleProps: styleProps
  }, other));
});
process.env.NODE_ENV !== "production" ? TableContainer.propTypes = {
  // ----------------------------- Warning --------------------------------
  // | These PropTypes are generated from the TypeScript type definitions |
  // |     To update them edit the d.ts file and run "yarn proptypes"     |
  // ----------------------------------------------------------------------

  /**
   * The content of the component, normally `Table`.
   */
  children: propTypes.node,

  /**
   * Override or extend the styles applied to the component.
   */
  classes: propTypes.object,

  /**
   * @ignore
   */
  className: propTypes.string,

  /**
   * The component used for the root node.
   * Either a string to use a HTML element or a component.
   */
  component: propTypes.elementType,

  /**
   * The system prop that allows defining system overrides as well as additional CSS styles.
   */
  sx: propTypes.object
} : void 0;

var tableSelectableSelector = function (state) {
    return state.reactTable.isSelectable;
};

function getStorageInitialState(localStorageKey) {
    var initState = {};
    var showColumn = localStorage.getItem(localStorageKey + '_columnOrder')
        ? JSON.parse(localStorage.getItem(localStorageKey + '_columnOrder'))
        : undefined;
    if (showColumn)
        initState['columnOrder'] = showColumn;
    var hideColumn = localStorage.getItem(localStorageKey + '_hiddenColumns')
        ? JSON.parse(localStorage.getItem(localStorageKey + '_hiddenColumns'))
        : undefined;
    if (hideColumn)
        initState['hiddenColumns'] = hideColumn;
    return initState;
}

function useMainResize() {
    var main = document.getElementById('main');
    var _a = useState({
        width: (main === null || main === void 0 ? void 0 : main.offsetWidth) || 0,
        height: (main === null || main === void 0 ? void 0 : main.offsetHeight) || 0,
    }), dimensions = _a[0], setDimensions = _a[1];
    useEffect(function () {
        var ro = new ResizeObserver_2(function (entries) {
            if (main) {
                var getDimensions = function () { return ({
                    width: (main === null || main === void 0 ? void 0 : main.offsetWidth) || 0,
                    height: (main === null || main === void 0 ? void 0 : main.offsetHeight) || 0,
                }); };
                setDimensions(getDimensions());
            }
        });
        if (main) {
            ro.observe(main);
        }
        return function () {
            if (ro) {
                ro.disconnect();
            }
        };
    }, [main]);
    return dimensions;
}

export { TableContainer as T, useSelector as a, getStorageInitialState as g, tableSelectableSelector as t, useMainResize as u };
//# sourceMappingURL=useMainResize-6722e68f.js.map
