import { _ as _extends, p as propTypes } from './index-8b5bbbf1.js';
import { _ as _objectWithoutPropertiesLoose } from './objectWithoutPropertiesLoose-6856ddd5.js';
import { forwardRef, createElement, useState, useEffect } from 'react';
import { c as clsx } from './clsx.m-6243b874.js';
import { g as generateUtilityClass, a as generateUtilityClasses, e as experimentalStyled, u as useThemeProps, c as composeClasses } from './experimentalStyled-c8814b05.js';
import { R as ResizeObserver_2 } from './ResizeObserver-da68295b.js';

function getTableContainerUtilityClass(slot) {
  return generateUtilityClass('MuiTableContainer', slot);
}
generateUtilityClasses('MuiTableContainer', ['root']);

const overridesResolver = (props, styles) => styles.root || {};

const useUtilityClasses = styleProps => {
  const {
    classes
  } = styleProps;
  const slots = {
    root: ['root']
  };
  return composeClasses(slots, getTableContainerUtilityClass, classes);
};

const TableContainerRoot = experimentalStyled('div', {}, {
  name: 'MuiTableContainer',
  slot: 'Root',
  overridesResolver
})({
  /* Styles applied to the root element. */
  width: '100%',
  overflowX: 'auto'
});
const TableContainer = /*#__PURE__*/forwardRef(function TableContainer(inProps, ref) {
  const props = useThemeProps({
    props: inProps,
    name: 'MuiTableContainer'
  });

  const {
    className,
    component = 'div'
  } = props,
        other = _objectWithoutPropertiesLoose(props, ["className", "component"]);

  const styleProps = _extends({}, props, {
    component
  });

  const classes = useUtilityClasses(styleProps);
  return /*#__PURE__*/createElement(TableContainerRoot, _extends({
    ref: ref,
    as: component,
    className: clsx(classes.root, className),
    styleProps: styleProps
  }, other));
});
process.env.NODE_ENV !== "production" ? TableContainer.propTypes = {
  // ----------------------------- Warning --------------------------------
  // | These PropTypes are generated from the TypeScript type definitions |
  // |     To update them edit the d.ts file and run "yarn proptypes"     |
  // ----------------------------------------------------------------------

  /**
   * The content of the component, normally `Table`.
   */
  children: propTypes.node,

  /**
   * Override or extend the styles applied to the component.
   */
  classes: propTypes.object,

  /**
   * @ignore
   */
  className: propTypes.string,

  /**
   * The component used for the root node.
   * Either a string to use a HTML element or a component.
   */
  component: propTypes.elementType,

  /**
   * The system prop that allows defining system overrides as well as additional CSS styles.
   */
  sx: propTypes.object
} : void 0;

function getStorageInitialState(localStorageKey) {
    var initState = {};
    var showColumn = localStorage.getItem(localStorageKey + '_columnOrder')
        ? JSON.parse(localStorage.getItem(localStorageKey + '_columnOrder'))
        : undefined;
    if (showColumn)
        initState['columnOrder'] = showColumn;
    var hideColumn = localStorage.getItem(localStorageKey + '_hiddenColumns')
        ? JSON.parse(localStorage.getItem(localStorageKey + '_hiddenColumns'))
        : undefined;
    if (hideColumn)
        initState['hiddenColumns'] = hideColumn;
    return initState;
}

function useMainResize() {
    var main = document.getElementById('main');
    var _a = useState({
        width: (main === null || main === void 0 ? void 0 : main.offsetWidth) || 0,
        height: (main === null || main === void 0 ? void 0 : main.offsetHeight) || 0,
    }), dimensions = _a[0], setDimensions = _a[1];
    useEffect(function () {
        var ro = new ResizeObserver_2(function (entries) {
            if (main) {
                var getDimensions = function () { return ({
                    width: (main === null || main === void 0 ? void 0 : main.offsetWidth) || 0,
                    height: (main === null || main === void 0 ? void 0 : main.offsetHeight) || 0,
                }); };
                setDimensions(getDimensions());
            }
        });
        if (main) {
            ro.observe(main);
        }
        return function () {
            if (ro) {
                ro.disconnect();
            }
        };
    }, [main]);
    return dimensions;
}

export { TableContainer as T, getStorageInitialState as g, useMainResize as u };
//# sourceMappingURL=useMainResize-7ae44bf0.js.map
